"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapMigration = void 0;
const typedi_1 = require("typedi");
const fs_1 = require("fs");
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
const config_1 = __importDefault(require("../../config"));
const constants_1 = require("../../constants");
const dsl_1 = require("../dsl");
const Logger_1 = require("../../Logger");
const NodeTypes_1 = require("../../NodeTypes");
const logger = (0, Logger_1.getLogger)();
const PERSONALIZATION_SURVEY_FILENAME = 'personalizationSurvey.json';
function loadSurveyFromDisk() {
    const userSettingsPath = n8n_core_1.UserSettings.getUserN8nFolderPath();
    try {
        const filename = `${userSettingsPath}/${PERSONALIZATION_SURVEY_FILENAME}`;
        const surveyFile = (0, fs_1.readFileSync)(filename, 'utf-8');
        (0, fs_1.rmSync)(filename);
        const personalizationSurvey = JSON.parse(surveyFile);
        const kvPairs = Object.entries(personalizationSurvey);
        if (!kvPairs.length) {
            throw new Error('personalizationSurvey is empty');
        }
        else {
            const emptyKeys = kvPairs.reduce((acc, [, value]) => {
                if (!value || (Array.isArray(value) && !value.length)) {
                    return acc + 1;
                }
                return acc;
            }, 0);
            if (emptyKeys === kvPairs.length) {
                throw new Error('incomplete personalizationSurvey');
            }
        }
        return surveyFile;
    }
    catch (error) {
        return null;
    }
}
let runningMigrations = false;
function logMigrationStart(migrationName) {
    if (constants_1.inTest)
        return;
    if (!runningMigrations) {
        logger.warn('Migrations in progress, please do NOT stop the process.');
        runningMigrations = true;
    }
    logger.debug(`Starting migration ${migrationName}`);
}
function logMigrationEnd(migrationName) {
    if (constants_1.inTest)
        return;
    logger.debug(`Finished migration ${migrationName}`);
}
const runDisablingForeignKeys = async (migration, context, fn) => {
    const { dbType, queryRunner } = context;
    if (dbType !== 'sqlite')
        throw new Error('Disabling transactions only available in sqlite');
    await queryRunner.query('PRAGMA foreign_keys=OFF');
    await queryRunner.startTransaction();
    try {
        await fn.call(migration, context);
        await queryRunner.commitTransaction();
    }
    catch (e) {
        try {
            await queryRunner.rollbackTransaction();
        }
        catch { }
        throw e;
    }
    finally {
        await queryRunner.query('PRAGMA foreign_keys=ON');
    }
};
function parseJson(data) {
    return typeof data === 'string' ? (0, n8n_workflow_1.jsonParse)(data) : data;
}
const dbType = config_1.default.getEnv('database.type');
const isMysql = ['mariadb', 'mysqldb'].includes(dbType);
const dbName = config_1.default.getEnv(`database.${dbType === 'mariadb' ? 'mysqldb' : dbType}.database`);
const tablePrefix = config_1.default.getEnv('database.tablePrefix');
const createContext = (queryRunner, migration) => ({
    logger,
    tablePrefix,
    dbType,
    isMysql,
    dbName,
    migrationName: migration.name,
    queryRunner,
    schemaBuilder: (0, dsl_1.createSchemaBuilder)(tablePrefix, queryRunner),
    nodeTypes: typedi_1.Container.get(NodeTypes_1.NodeTypes),
    loadSurveyFromDisk,
    parseJson,
    escape: {
        columnName: (name) => queryRunner.connection.driver.escape(name),
        tableName: (name) => queryRunner.connection.driver.escape(`${tablePrefix}${name}`),
        indexName: (name) => queryRunner.connection.driver.escape(`IDX_${tablePrefix}${name}`),
    },
    runQuery: async (sql, unsafeParameters, safeParameters) => {
        if (unsafeParameters) {
            const [query, parameters] = queryRunner.connection.driver.escapeQueryWithParameters(sql, unsafeParameters, safeParameters !== null && safeParameters !== void 0 ? safeParameters : {});
            return queryRunner.query(query, parameters);
        }
        else {
            return queryRunner.query(sql);
        }
    },
    runInBatches: async (query, operation, limit = 100) => {
        let offset = 0;
        let batchedQuery;
        let batchedQueryResults;
        if (query.trim().endsWith(';'))
            query = query.trim().slice(0, -1);
        do {
            batchedQuery = `${query} LIMIT ${limit} OFFSET ${offset}`;
            batchedQueryResults = (await queryRunner.query(batchedQuery));
            await operation([...batchedQueryResults]);
            offset += limit;
        } while (batchedQueryResults.length === limit);
    },
    copyTable: async (fromTable, toTable, fromFields, toFields, batchSize) => {
        const { driver } = queryRunner.connection;
        fromTable = driver.escape(`${tablePrefix}${fromTable}`);
        toTable = driver.escape(`${tablePrefix}${toTable}`);
        const fromFieldsStr = (fromFields === null || fromFields === void 0 ? void 0 : fromFields.length)
            ? fromFields.map((f) => driver.escape(f)).join(', ')
            : '*';
        const toFieldsStr = (toFields === null || toFields === void 0 ? void 0 : toFields.length)
            ? `(${toFields.map((f) => driver.escape(f)).join(', ')})`
            : '';
        const total = await queryRunner
            .query(`SELECT COUNT(*) AS count FROM ${fromTable}`)
            .then((rows) => rows[0].count);
        batchSize = batchSize !== null && batchSize !== void 0 ? batchSize : 10;
        let migrated = 0;
        while (migrated < total) {
            await queryRunner.query(`INSERT INTO ${toTable} ${toFieldsStr} SELECT ${fromFieldsStr} FROM ${fromTable} LIMIT ${migrated}, ${batchSize}`);
            migrated += batchSize;
        }
    },
});
const wrapMigration = (migration) => {
    const { up, down } = migration.prototype;
    Object.assign(migration.prototype, {
        async up(queryRunner) {
            logMigrationStart(migration.name);
            const context = createContext(queryRunner, migration);
            if (this.transaction === false) {
                await runDisablingForeignKeys(this, context, up);
            }
            else {
                await up.call(this, context);
            }
            logMigrationEnd(migration.name);
        },
        async down(queryRunner) {
            if (down) {
                const context = createContext(queryRunner, migration);
                if (this.transaction === false) {
                    await runDisablingForeignKeys(this, context, up);
                }
                else {
                    await down.call(this, context);
                }
            }
        },
    });
};
exports.wrapMigration = wrapMigration;
//# sourceMappingURL=migrationHelpers.js.map